import pytest
from dingus import Dingus
from bunch import Bunch

from jobtriggers import bodhi_msg


@pytest.mark.usefixtures('prepare')
class TestBodhiJobTrigger():
    @pytest.fixture
    def prepare(self, monkeypatch):
        self.ref_updatetitle = 'foo'
        self.ref_dist_tag = 'f18'
        self.ref_koji_tag = '%s-updates' % self.ref_dist_tag
        self.ref_nvr = "gnome-settings-daemon-3.6.1-1.fc18"
        self.ref_topic = 'org.fedoraproject.prod.bodhi.update.request.stable'

        self.ref_tasks = ['bodhi_task1', 'bodhi_task2']
        self.ref_validarches = ['i386', 'x86_64']
        self.ref_blacklist_releases= ['el5', 'el6']

        self._create_msg(self.ref_updatetitle, self.ref_dist_tag, self.ref_nvr, self.ref_topic)

        stub_hub = Bunch(config=Bunch(get=0))
        self.helper = bodhi_msg.BodhiJobTrigger(stub_hub)

        self.helper.trigger = Dingus()

        monkeypatch.setattr(bodhi_msg.utils, 'get_arches', lambda *a, **k: self.ref_validarches)

        bodhi_msg.config.bodhi_tasks = self.ref_tasks
        bodhi_msg.config.bodhi_to_koji_tag_tasks = self.ref_tasks
        bodhi_msg.config.valid_arches = self.ref_validarches
        bodhi_msg.config.blacklist_releases = self.ref_blacklist_releases
        bodhi_msg.config.job_logging = False

    def _create_msg(self, ref_updatetitle, ref_dist_tag, ref_nvr, ref_topic):
        self.ref_message = Bunch(body='{"msg": {"agent": "lmacken",\
                                        "update": {"submitter": "lmacken",\
                                                   "title": "%s",\
                                                   "release": {"dist_tag": "%s"},\
                                                   "builds": [{"nvr": "%s"}]}},\
                                        "topic": "%s"}' %
                                        (ref_updatetitle, ref_dist_tag, ref_nvr, ref_topic))

    def test_consume(self):
        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert len(trigger_calls) == 8

        assert trigger_calls[0][1] == (self.ref_updatetitle, 'bodhi_id', self.ref_tasks[0], self.ref_validarches[0])
        assert trigger_calls[1][1] == (self.ref_updatetitle, 'bodhi_id', self.ref_tasks[0], self.ref_validarches[1])
        assert trigger_calls[2][1] == (self.ref_updatetitle, 'bodhi_id', self.ref_tasks[1], self.ref_validarches[0])
        assert trigger_calls[3][1] == (self.ref_updatetitle, 'bodhi_id', self.ref_tasks[1], self.ref_validarches[1])
        assert trigger_calls[4][1] == (self.ref_koji_tag, 'koji_tag', self.ref_tasks[0], self.ref_validarches[0])
        assert trigger_calls[5][1] == (self.ref_koji_tag, 'koji_tag', self.ref_tasks[0], self.ref_validarches[1])
        assert trigger_calls[6][1] == (self.ref_koji_tag, 'koji_tag', self.ref_tasks[1], self.ref_validarches[0])
        assert trigger_calls[7][1] == (self.ref_koji_tag, 'koji_tag', self.ref_tasks[1], self.ref_validarches[1])

    def test_consume_notwatched_topic(self):
        ref_topic = 'org.fedoraproject.pro.bodhi.update.request.randomtopic'
        self._create_msg(self.ref_updatetitle, self.ref_dist_tag, self.ref_nvr, ref_topic)

        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert trigger_calls == []

    def test_consume_blacklisted_release(self):
        ref_nvr = 'gnome-settings-daemon-3.6.1-1.el5'
        self._create_msg(self.ref_updatetitle, self.ref_dist_tag, ref_nvr, self.ref_topic)

        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert trigger_calls == []

    def test_consume_kojitag_testing(self):
        ref_topic = 'org.fedoraproject.pro.bodhi.update.request.testing'
        self._create_msg(self.ref_updatetitle, self.ref_dist_tag, self.ref_nvr, ref_topic)

        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert trigger_calls[4][1][0] == self.ref_koji_tag + '-testing'
